import { Component } from '@angular/core';
import { DataService } from 'src/services/data/data.service';
import { NgxSmartModalService } from 'ngx-smart-modal';
import { DetailsService } from 'src/services/details/details.service';
import { CdkDragDrop, moveItemInArray } from '@angular/cdk/drag-drop';
import { Spell } from 'src/interfaces/spell';
import { SpellDetailsComponent } from './spell-details/spell-details.component';

@Component({
  selector: 'spell-table',
  templateUrl: './spell-table.component.html',
  styleUrls: ['./spell-table.component.scss'],
})
export class SpellTableComponent {
  public constructor(
    public dataAccessor: DataService,
    public ngxSmartModalService: NgxSmartModalService,
    public detailsAccessor: DetailsService
  ) {}

  public spellAttackBonus: string = '0';
  public spellSaveDC: number = 0;
  private spellcastingAttribute: string | undefined;
  private spellcastingAttributeModifier: number = 0;
  private proficiencyBonus: number = 2;

  public spells!: Spell[];

  public attributes: any = {
    strength: 'STR',
    dexterity: 'DEX',
    constitution: 'CON',
    intelligence: 'INT',
    wisdom: 'WIS',
    charisma: 'CHA',
  };

  public areas: any = {
    cone: 'Kegel',
    sphere: 'Kugel',
    circle: 'Kreis',
    line: 'Linie',
    square: 'Quadrat',
    cube: 'Würfel',
  };

  public ngOnInit(): void {
    this.spells = this.dataAccessor.getSpells();
    this.subscribeToData();
  }

  public openDetailsPanel(index: number): void {
    this.detailsAccessor.openPanel(SpellDetailsComponent, {
      spell: this.spells[index],
      modifiers: {
        attackBonus: this.spellAttackBonus,
        saveDC: this.spellSaveDC,
      },
    });
    const resultSubscription = this.detailsAccessor.result$.subscribe(
      (result) => {
        console.log(result);
        if (result.state === 'delete') {
          this.deleteSpell(index);
        } else if (result.state === 'update') {
          this.openSpellModificationModal(index);
        }
        resultSubscription.unsubscribe();
      }
    );
  }

  public openSpellModal(): void {
    this.ngxSmartModalService.getModal('spellModal').open();
  }

  public openSpellModificationModal(index: number): void {
    this.ngxSmartModalService
      .getModal('spellModal')
      .setData({ spell: this.spells[index], index: index });
    this.ngxSmartModalService.getModal('spellModal').open();
  }

  public addSpell(spell: Spell) {
    this.spells.push(spell);
    this.updateSpellsInDatabase();
  }

  public updateSpell(spell: Spell, index: number): void {
    this.spells[index] = spell;
    this.updateSpellsInDatabase();
  }

  public deleteSpell(index: number): void {
    this.spells.splice(index, 1);
    this.updateSpellsInDatabase();
  }

  // utils

  public dropSpells(event: CdkDragDrop<string[]>): void {
    moveItemInArray(this.spells, event.previousIndex, event.currentIndex);
    this.updateSpellsInDatabase();
  }

  public updateSpellsInDatabase(): void {
    this.dataAccessor.setSpells(this.spells);
  }

  private computeSpellAttackBonusAndSaveDC(): void {
    let attackBonus =
      this.spellcastingAttributeModifier + this.proficiencyBonus;
    this.spellSaveDC = 8 + attackBonus;
    if (attackBonus >= 0) {
      this.spellAttackBonus = '+' + attackBonus;
    } else {
      this.spellAttackBonus = attackBonus.toString();
    }
  }

  private subscribeToData(): void {
    // TODO: this.dataAccessor.getSpellcastingAttribute() oder so
    this.spellcastingAttribute = 'intelligence';

    this.dataAccessor.proficiency$.subscribe((value) => {
      this.proficiencyBonus = value.value;
      if (this.spellcastingAttribute) {
        this.computeSpellAttackBonusAndSaveDC();
      }
    });
    // TODO: Modify depending on the actual spellcasting attribute
    this.dataAccessor.intelligence$.subscribe((value) => {
      this.spellcastingAttributeModifier = Math.floor((value.value - 10) / 2);
      this.computeSpellAttackBonusAndSaveDC();
    });
  }
}
