import { Component, EventEmitter, Output } from '@angular/core';
// import { Observable } from 'rxjs';
import { DataService } from 'src/services/data/data.service';
import { Attribute } from 'src/interfaces/attribute';
import { Weapon } from 'src/interfaces/weapon';
// import { Damage } from 'src/interfaces/damage';
import { CdkDragDrop, moveItemInArray } from '@angular/cdk/drag-drop';
import { DetailsService } from 'src/services/details/details.service';
import { NgxSmartModalService } from 'ngx-smart-modal';
import { WeaponDetailsComponent } from './weapon-details/weapon-details.component';

@Component({
  selector: 'weapon-table',
  templateUrl: './weapon-table.component.html',
  styleUrls: ['./weapon-table.component.scss'],
})
export class WeaponTableComponent {
  public constructor(
    public dataAccessor: DataService,
    public detailsAccessor: DetailsService,
    public ngxSmartModalService: NgxSmartModalService
  ) {}

  @Output() public createNewWeapon: EventEmitter<void> =
    new EventEmitter<void>();

  @Output() public updateWeaponEvent: EventEmitter<any> =
    new EventEmitter<any>();

  public weapons!: Weapon[];
  public damageModifiers: string[] = [
    '0',
    '0',
    '0',
    '0',
    '0',
    '0',
    '0',
    '0',
    '0',
    '0',
  ];
  public isMonk: boolean = false;
  public strengthValue: number = 0;
  public dexterityValue: number = 0;

  public ngOnInit(): void {
    this.dataAccessor.strength$.subscribe((newValue: Attribute) => {
      this.strengthValue = newValue.value;
      if (this.weapons) {
        this.calculateDamageModifierArray();
      }
    });
    this.dataAccessor.dexterity$.subscribe((newValue: Attribute) => {
      this.dexterityValue = newValue.value;
      if (this.weapons) {
        this.calculateDamageModifierArray();
      }
    });
    this.weapons = this.dataAccessor.getWeapons();
    this.calculateDamageModifierArray();
  }

  /**
   * Claculates the whole damage modifier array.
   * Calls the single damage modifier calculation for each weapon.
   */
  private calculateDamageModifierArray(): void {
    this.weapons.forEach((_, index) => {
      this.calculateSingleDamageModifier(index);
    });
  }

  private calculateSingleDamageModifier(index: number): void {
    let value: number;
    let bonus: number = 0;
    const weapon: Weapon = this.weapons[index];
    if (this.isMonk || weapon.isFinesse) {
      value = Math.max(this.strengthValue, this.dexterityValue);
    } else if (weapon.isRanged) {
      value = this.dexterityValue;
    } else {
      value = this.strengthValue;
    }
    if (weapon.isMagical) {
      bonus = weapon.magicBonus!;
    }
    this.damageModifiers[index] = this.calculateModifier(value, bonus);
  }

  public dropWeapons(event: CdkDragDrop<string[]>): void {
    moveItemInArray(this.weapons, event.previousIndex, event.currentIndex);
    moveItemInArray(
      this.damageModifiers,
      event.previousIndex,
      event.currentIndex
    );
    this.updateWeaponInDatabase();
  }

  public updateWeaponInDatabase(): void {
    this.dataAccessor.setWeapons(this.weapons);
  }

  public openWeaponModal(): void {
    this.ngxSmartModalService.getModal('weaponModal').open();
  }

  public addWeapon(weapon: Weapon) {
    this.weapons.push(weapon);
    this.calculateDamageModifierArray();
    this.updateWeaponInDatabase();
  }

  public openDetailsPanel(index: number): void {
    this.detailsAccessor.openPanel(WeaponDetailsComponent, {
      weapon: this.weapons[index],
      damageModifier: this.damageModifiers[index],
    });
    const resultSubscription = this.detailsAccessor.result$.subscribe(
      (result) => {
        console.log(result);
        if (result.state === 'delete') {
          this.deleteWeapon(index);
        } else if (result.state === 'update') {
          this.openWeaponModificationModal(index);
        }
        resultSubscription.unsubscribe();
      }
    );
  }

  public openWeaponModificationModal(index: number): void {
    this.ngxSmartModalService
      .getModal('weaponModal')
      .setData({ weapon: this.weapons[index], index: index });
    this.ngxSmartModalService.getModal('weaponModal').open();
  }

  public updateWeapon(weapon: Weapon, index: number): void {
    this.weapons[index] = weapon;
    this.calculateSingleDamageModifier(index);
    this.updateWeaponInDatabase();
  }

  public deleteWeapon(index: number): void {
    this.weapons.splice(index, 1);
    this.calculateDamageModifierArray();
    this.updateWeaponInDatabase();
  }

  ////// Calculation Helpers

  private calculateModifier(attributeValue: number, bonus: number): string {
    let mod: number = Math.floor((attributeValue - 10) / 2) + bonus;
    if (mod > 0) {
      return '+' + mod;
    } else {
      return mod.toString();
    }
  }
}
